/**
 * CapacitorJS plugin to update the web contents of an app from a remote content server.
 *
 * @example
 * ```js
 * import { AppUpdater } from '@objekt/capacitor-app-updater';
 *
 * // Check for app updates on the remote content server.
 * const didUpdate = await AppUpdater.sync("https://your-web-server-url", 1000*60*60); // Only check once every 60 minutes.
 *
 * // Stop processing if there was an update, as the updated would have triggered a page reload.
 * if (didUpdate) {
 * 	return;
 * }
 *
 * // Load the app shell.
 * // ... e.g. await import('./src/AppShell.js');
 * ```
 */
export declare const AppUpdater: {
    /**
     * Syncs the online web app to the native app shell.
     *
     * Note: this function triggers a browser reload if the app was updated successfully to point to the new release.
     *
     * @param webServerURL - The URL of the online web server.
     * @param checkDelay - The amount of time to allow between update checks. Defaults to 60 minutes.
     *
     * @returns True, if the app was updated, otherwise false.
     */
    sync: (webServerURL: string, checkDelay?: number) => Promise<boolean>;
};
//# sourceMappingURL=AppUpdater.d.ts.map